-- Window collection
Windows = {}

-- Check for reload data
ReloadStateData = Turbine.PluginData.Load(Turbine.DataScope.Server, "PrimeReloadData") 

-- Setup player and party data
Player = Turbine.Gameplay.LocalPlayer.GetInstance();
PlayerName = Player:GetName()
Name = Player:GetName();
Party = Player:GetParty();
if Party ~= nil then
    GroupSize = Party:GetMemberCount()
else
    GroupSize = 1
end

-- Create parser
ChatEventHandler = ChatHandler()

-- Handle lootlock data
LootLocks = Turbine.PluginData.Load(Turbine.DataScope.Server, "PrimeLootLocks")
if LootLocks == nil then
    LootLocks = {}
end

--Build current party to not break the plugin
Group = {}
GroupIDToName = {}
PlayerEffectsHandler = PlayerEffectEvents()
GroupEffectsHandler = GroupEffectEvents()

--SetGroup() -- TODO ADD NEW BINDING
--SetEffects()

-- Create the global upate handler
Updater = UpdateHandler()
Windows["Updater"] = Updater

-- Countdown window
countdownWindow = CountdownWindow()
Windows["Countdown"] = countdownWindow

-- Combat timer
if Settings.CTEnabled then 
    combatTimer = CombatWindow() 
    Windows["Combat Timer"] = combatTimer
end

-- Alert, dnf tracker and effects
if Settings.AEnabled then 
    messageWindow = AlertWindow() 
    Windows["Alert Window"] = messageWindow
end 

if Settings.AEnabled then 
    rezTracker = DNFTracker() 
    Windows["DNF Tracker"] = rezTracker
end

if Settings.EEnabled then 
    effectsWindow = EffectsWindow() 
    Windows["Effect Window"] = effectsWindow
end 

-- Skill & boss timers
if Settings.SEnabled then 
    skillCounters = SkillCounters()
    Windows["Boss Timers"] = skillCounters
end 

if Settings.SPEnabled then 
    skillTimers = SkillTimers(0)
    Windows["Cooldown Timers"] = skillTimers
end 

if Settings.SPEnabled then 
    activeTimers = SkillTimers(1) 
    Windows["Active Timers"] = activeTimers
end

-- Note Window
noteWindow = NoteWindow()
Windows["Note Window"] = noteWindow

-- Leader & Log Window
if Settings.LWEnabled then 
    leaderWindow = LeaderWindow() 
    Windows["Leader Window"] = leaderWindow
end

if Settings.LEnabled then 
    logWindow = LogWindow() 
    Windows["Log Window"] = logWindow
end 

-- Loot, Stat window & dungeon window
if Settings.LOEnabled then 
    lootWindow = LootWindow() 
    Windows["Loot Window"] = lootWindow
end
Statwindow = nil --Stats()
DungeonWindow = nil --DungeonLog()

-- Options button & reload windows
reloadWindow = ReloadWindow()
Windows["Reload Window"] = reloadWindow

optionsButton = OptionsButton()
Windows["Options Button"] = optionsButton

MasterOptions = nil

-- Skill window
skillWindow = SkillWindow()
Windows["Skill Window"] = skillWindow

-- Improved ready check windows
ReadyCheck = ReadyWindow()
Windows["Ready Check"] = ReadyCheck

RcWindow = ReadyCheckWindow()
Windows["Ready Check Window"] = RcWindow

RCButtonWindow = ReadyButtonWindow()
Windows["Ready Check Button Window"] = RCButtonWindow

RButton = ReloadButton()
Windows["Reload Button"] = RButton


-- Sound player
Sounds = SoundPlayer()
Windows["Sound Player"] = Sounds

-- Swap window
SwappyWarner = SwapWarner()
Windows["Swap Warner"] = SwappyWarner

-- Instance Hanlder
InstanceController = InstanceHub()
Windows["Instance Controller"] = InstanceController

-- Skill Alert Timers
if Settings.SKAEnabled or Settings.SKTEnabled then 
    PVPCDTracker = PvpTimers()
    Windows["PVP CD Tracker"] = PVPCDTracker
end

-- Target Window
if Settings.TEnabled then 
    if GroupSize <= 3 or Settings.TOverride then
        Target = TargetWindow()
        Windows["Target Window"] = Target
    else
        ChatOutput("Target window disabled due to group size! You can override this in the options.")
    end
end

-- Target Box
if Settings.TBEnabled then 
    targetBox = TargetBox()
    Windows["Target Box"] = targetBox
end

-- Garbage Collection handler
GCHandler = GarbageHandler()
Windows["GC Handler"] = GCHandler

-- Create buff windows from saved settings
HandleBuffWindows()

-- Set global event handlers
InitiateHandlers()

-- Handle reloading when notes and countdown exists
if ReloadStateData ~= nil then
    if ReloadStateData.Note ~= nil then
        if ReloadStateData.Note ~= "" then 
            noteWindow.noteWindow:SetText(ReloadStateData.Note)
            ChatOutput("<rgb=#004d00>Note window repopulated after restart, type /prime noteClear to clear the note window if this is needed.</rgb>")
        end
    end
    local cd = ReloadStateData.Countdown
    if ReloadStateData.Countdown ~= nil and cd > 0 then
        countdownWindow:Countdown(cd)
    end
end

-- Load the plugin manager options
import "PrimePlugins.RaidTools.Options"

-- Setup all group events
--SetTriggers() -- TODO ADD NEW BINDING

-- Initialize commands
SetCommands()

-- Create skill overlays from data
CreateSkillBarFromSave()

-- Handle first time use current version
Setup = nil
if Settings.FirstTime then
    Setup = FirstTimerOptions()
    Windows["FirstTimeSetup"] = Setup
end

-- Import test env
--import "PrimePlugins.RaidTools.TestFile"

-- Set back to normal state
Settings.ReloadData = false

WindowHandler = class(Turbine.UI.Control)
function WindowHandler:Constructor()
    Turbine.UI.Control.Constructor(self);
    self:SetWantsKeyEvents(true)

    self.Hide = false

    self.ShowState = {}

    self.KeyDown = function(sender, args)
        if args.Action == 268435635 then
            if self.Hide then 
                self:ShowWindows()
            else
                self:HideWindows()
            end
        end
    end
end

function WindowHandler:HideWindows()

end

function WindowHandler:ShowWindows()
    self.Hide = false
    for _, value in pairs(self.ShowState) do
        value:SetVisible(true)
    end
    self.ShowState = {}
end

function WindowHandler:HideWindows()
    self.Hide = true
    for key, value in pairs(Windows) do
        if value:IsVisible() then
            self.ShowState[key] = value
        end
        value:SetVisible(false)
    end
end

WindowsHandlerController = WindowHandler()

import "PrimePlugins.RaidTools.MoveHandler"